#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::TestDeveloperAuthentification
# state        public
#----------------------------------------------------------------------------------------
# description  test the behavior of the DeveloperAuthentification package
#========================================================================================
package TestDeveloperAuthentification;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::DeveloperAuthentification qw ( SetAuthorKeys GetAuthorKeys );
use Test::More;

my $EPAGES_CONFIG_RETURN_VALUE;
sub Main {
    OverwriteEpagesConfigCallsForTests();
    AuthorKeys();
    Test_GetAuthorKeysIfUndefined();
    done_testing();
}

sub AuthorKeys {
    my $subName = (caller(0))[3];
    SetAuthorKeys('abc','cde','def');
    my $ExpectedResult = {
          'DEVID'  => 'abc',
          'CERTID' => 'def',
          'APPID'  => 'cde'
        };
    # this tests fails if there is a value in the epages.config -> EncryptedDevelopmentKeys remove
    is_deeply(GetAuthorKeys(), $ExpectedResult, "$subName - test if we can read encryped values from config file - hash with authkeys");
}

sub Test_GetAuthorKeysIfUndefined {
    my $subName = (caller(0))[3];
    $EPAGES_CONFIG_RETURN_VALUE = undef;
    is(GetAuthorKeys(),undef,"$subName - behavoir if there is now value in epages config file - undef");
}

sub OverwriteEpagesConfigCallsForTests {
    ## no critic ( ProtectPrivateSubs)
    * DE_EPAGES::DHLIntraship::API::DeveloperAuthentification::_SetValueToEpagesConfig = sub {
        my ( $EncryptedDevelopmentKeys ) = @_;
        $EPAGES_CONFIG_RETURN_VALUE = $EncryptedDevelopmentKeys;
    };
    * DE_EPAGES::DHLIntraship::API::DeveloperAuthentification::_GetValueFromEpagesConfig = sub {
        return $EPAGES_CONFIG_RETURN_VALUE;
    };
    ## use critic ( ProtectPrivateSubs)
}

RunScript( Sub => \&Main );