/*
 * @Copyright   Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module      ckeditor/plugins/epgadget/plugin
 *
 * @revision    $$
 */
/*global define, window, de_epages*/
define('ckeditor/plugins/epgadget/plugin', [
    'jquery',
    'ep',
    'ckeditor/ckeditor',
    'util/json',
    'de_epages/externalcontent/gadgetdialog/constants',
    '$dict!ep/dict'
], function ($, ep, CKEDITOR, JSON, CONSTANTS, epDict) {

    'use strict';

    var selectedNode, tGadget = epDict.translate('Gadget'),
        createImgSrc = function (data) {
            var srcValue = ep.config.iconsRoot + '/gadget',
                opt;
            if (data) {
                srcValue += '_' + data.alias;
                opt = data.options;
                if (opt && data.alias === 'maps') {
                    if (opt.layout) {
                        srcValue += '_' + opt.layout;
                    }
                    if (opt.type) {
                        srcValue += '_' + opt.type;
                    }
                }
            }
            srcValue += '.png';
            return srcValue;
        },
        topRequire = window.require,
        gadgetClass = CONSTANTS.gClassname,
        pluginName = 'epgadget';

    // Work-around for opening dialog outside of iframe,
    // when CKEditor is instanciated on the SF inside an iframe in MBO.
    if (window.top !== window.self) {
        try {
            topRequire = window.top.require;
        } catch (error) {}
    }


    CKEDITOR.plugins.add(pluginName, {

        init: function (editor) {

            editor.addCommand(pluginName, {
                exec: function (editor, alias) {
                    var selectedElement, dataAttribute, gadgetOptions = {
                        callback: function (data) {
                            var imgSrc = createImgSrc(data),
                                element = new CKEDITOR.dom.element('div'),
                                dataString = JSON.stringify(data),
                                fakeElement;

                            element.setAttributes({
                                'class': gadgetClass,
                                'data-ep-gadget': dataString
                            });
                            // Create *fakeElement* which is used as placeholder.
                            fakeElement = editor.createFakeElement(element, gadgetClass, 'div');
                            fakeElement.setAttributes({
                                'alt': tGadget,
                                //'Gadget',
                                'title': tGadget,
                                //'Gadget',
                                'data-ep-gadget': dataString,
                                'contenteditable': false,
                                'style': 'max-width:100%;display:block;margin-bottom:10px;',
                                'src': imgSrc
                            });

                            // Remove selected element if gadget was selected before.
                            if (selectedNode) {
                                selectedElement = selectedNode.getSelectedElement();
                                if (selectedElement) {
                                    selectedElement.remove();
                                }

                            }
                            editor.insertElement(fakeElement);
                        }
                    };
                    // Check if gadget is selected and get gadget data from *fakeElement*.
                    selectedNode = editor.getSelection();
                    if (selectedNode) {
                        selectedElement = selectedNode.getSelectedElement();
                        if (selectedElement) {
                            dataAttribute = selectedElement.getAttribute('data-ep-gadget');
                            if (dataAttribute) {
                                gadgetOptions.gadget = JSON.parse(dataAttribute);
                            }
                        }
                    }

                    if (alias && alias === 'rss' && !gadgetOptions.gadget) {
                        gadgetOptions.gadget = {
                            alias: 'rss'
                        };
                    }

                    topRequire(['de_epages/externalcontent/gadgetdialog'], function (gadgetsdialog) {
                        // with EPG-28229 we removed the types "profile" and "search"
                        // but it is possible that merchants still have these types in their shop
                        // so we will send them to the standard twitter gadget, when they edit the old profile/search-twitter gadget.
                        if (gadgetOptions.gadget && gadgetOptions.gadget.alias === 'twitter' && (gadgetOptions.gadget.options.type === 'profile' || gadgetOptions.gadget.options.type === 'search')) {
                            gadgetOptions.gadget.options = {};
                        }
                        gadgetsdialog.open(gadgetOptions);
                    });
                }
            });

            editor.ui.addButton(pluginName, {
                label: tGadget,
                command: pluginName,
                icon: this.path + 'images/gadget.png',
                onRender: function () {
                    // Highlight gadget button on gadget selection.
                    editor.on('selectionChange', function (ev) {
                        this.setState(CKEDITOR.TRISTATE_OFF);

                        if (ev.data && ev.data.selection && ev.data.selection.getSelectedElement() !== null && ev.data.selection.getSelectedElement().hasClass(gadgetClass)) {
                            this.setState(CKEDITOR.TRISTATE_ON);
                        }
                    }, this);
                }
            });

            editor.on('doubleclick', function (ev) {
                if (ev.data && ev.data.element && ev.data.element.hasClass(gadgetClass)) {
                    editor.execCommand(pluginName);
                }
            });

        },

        afterInit: function (editor) {
            var dataProcessor = editor.dataProcessor,
                dataFilter = dataProcessor && dataProcessor.dataFilter;

            if (dataFilter) {
                dataFilter.addRules({
                    elements: {
                        div: function (element) {
                            // Select gadget and replace it with *fakeElement*.
                            if (element.attributes['class'] === gadgetClass) {
                                var fakeElement = editor.createFakeParserElement(element, gadgetClass, 'div', false),
                                    dataString = $('<div/>').html(element.attributes['data-ep-gadget']).text(),
                                    data = JSON.parse(dataString),
                                    opt = data.options,
                                    imgSrc = createImgSrc(data);
                                // We have to use fakeElement.attributes here because there is no setAttributes method for a FakeParserElement - very strange.
                                fakeElement.attributes.title = tGadget;
                                fakeElement.attributes.alt = tGadget;
                                fakeElement.attributes.style = 'max-width:100%;display:block;margin-bottom:10px;';
                                fakeElement.attributes.src = imgSrc;
                                fakeElement.attributes.contenteditable = false;
                                fakeElement.attributes['data-ep-gadget'] = dataString;
                                return fakeElement;
                            }
                        }
                    }
                });
            }
        }
    });

    return CKEDITOR;

});