/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.externalcontent.ui.cropslidedialog
 *
 * @revision		$$
 */

define("de_epages/externalcontent/ui/cropslidedialog", [
	"jquery",
	"ep",
	"de_epages",
	"fabric",
	"$dict!de_epages/externalcontent/dictionary",
	"$tmpl!de_epages/externalcontent/ui/cropslidedialog",

	"jquery/ui/widget",
	"ep/ui/dialog"
], function ($, ep, de_epages, fabric, dict, tmplCropslidedialog) {

	/*
	 * @dictionary		de_epages.externalcontent.gadgets
	 *
	 * @translation		{RepositionYourImage}
	 *					{ImageTooSmall}
	 */

	// The actual widget.
	$.widget('ui.externalcontentUiCropslidedialog', {

		options: {
			'width': 1050,
			'height': 350,
			'imageOptions': {},
			'dialogOptions': {},
			'canvasOptions': {},
			'url': '',
			'headerInfoText': ''
		},

		_create: function () {
			var self = this,
				o = self.options;

			$.extend(self, $.tmplItem(tmplCropslidedialog(o), 'elements'));

			ep(self.element.append(self.header, self.canvas))
				.uiDialog(o.dialogOptions);

			self.fabricCanvas = new fabric.Canvas(self.canvas[0], o.canvasOptions);

			fabric.Image.fromURL(o.url, $.proxy(self, '_onImageLoad'), o.imageOptions);
		},

		_setOption: function (key, value) {
			if (key === 'headerInfoText') {
				this.header.html(value);
			}

			return this._superApply(arguments);
		},

		getFabricCanvas: function () {
			return this.fabricCanvas;
		},

		getFabricImage: function () {
			return this.fabricImage;
		},

		_onImageLoad: function (image) {
			var self = this,
				o = self.options;

			// Store *image* for later access through *getFabricImage* method.
			self.fabricImage = image;

			// We do not allow scaling and rotating.
			image.hasControls = false;

			// Do not draw borders on image.
			image.hasBorders = false;

			// Now let's see which *setting* we have here.
			if (image.width < (o.width + 1)) {
				if (image.height > (o.height)) {
					image.lockMovementX = true;
					self._setOption('headerInfoText', dict.translate('{RepositionYourImage}'));
				}
				else {
					image.lockMovementX = image.lockMovementY = true;
					self.fabricCanvas.moveCursor = self.fabricCanvas.hoverCursor = self.fabricCanvas.defaultCursor;
					self._setOption('headerInfoText', dict.translate('{ImageTooSmall}'));
				}
			}
			else {
				if (image.height < (o.height + 1)) {
					image.lockMovementY = true;
					self._setOption('headerInfoText', dict.translate('{RepositionYourImage}'));
				}
				else {
					if (image.width > (3 * image.height - 1)) {
						image.scaleToHeight(o.height);
						image.lockMovementY = true;
						self._setOption('headerInfoText', dict.translate('{RepositionYourImage}'));
					}
					else {
						image.scaleToWidth(o.width);
						image.lockMovementX = true;
						self._setOption('headerInfoText', dict.translate('{RepositionYourImage}'));
					}
				}
			}

			// Center image.
			image.set(self.fabricCanvas.getCenter());

			// Add image to *canvas*.
			self.fabricCanvas.add(image);
		},

		destroy: function () {
			var self = this;

			// Remove all traces of the widget.
			self.canvas.remove();

			ep(self.element)
				.uiDialog('destroy');

			return self._superApply(arguments);
		}

	});

	return de_epages;

});