/*
 * @copyright	© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module		de_epages.externalcontent.ui.maps
 *
 * @revision    $Revision: 1.2 $
 */
/*jslint nomen: true, browser: true*/
/*global define*/
define('de_epages/externalcontent/ui/maps', [
	'jquery/ui/widget',
	'ep',
	'de_epages',

	'jquery/tmpl',
	'jquery/uid'
], function ($, ep, de_epages) {
	'use strict';
    // Gagdet class name
    var gadgetClassName = '',

        // Container template id
        tmplContainerId = 'de_epages.externalcontent.ui.maps',

        // Cache of ajax deferreds by url
        ajaxCache = {};

    // Render and cache template.
    $.template(tmplContainerId, '<div style="width: ${width}px; height: ${height}px;{{if type === "bing"}} position: relative;{{/if}}"></div>');

    // Define widget
    $.widget('ui.externalcontentUiMaps', {
        options: {
            type: 'google',
            controls: true,
            fixedView: false,
            width: 500,
            height: 300,
            lat: 50.9287910,
            lng: 11.5846220,
            zoom: 5,
            title: ''
        },

        _create: function () {
            var self = this,
                type = self.options.type,
                api = self.api = self[type] && self[type].render ? self[type] : self.google; // Use google as default fallback
            // Load API async
            ((api.load && api.load.call(self)) || $.Deferred().resolve()).done(function (data) {
                // Add ui class name
                self.element.addClass(gadgetClassName);

                // Start rendering
                api.render.call(self, data);
            });
        },

        // Google Map load and render API
        google: {
            // Load API and return deferred object or false
            load: function () {
                var url = 'https://www.google.com/jsapi';

                // API already loaded
                if (window.google && window.google.load) {
                    return;
                }

                // Request already started
                if (ajaxCache[url]) {
                    return ajaxCache[url];
                }

                // Request API
                return (ajaxCache[url] = ep.ajax({
                    url: url,
                    dataType: 'script',
                    cache: true
                }));
            },
            // Render map using loaded api
            render: function () {
                var self = this,
                    o = self.options;

                // Load Google Maps API.
                window.google.load('maps', '3', {
                    other_params: 'sensor=false',
                    callback: function () {
                        var centerMarker,
                            center;
                        // Init *container*.
                        self.container = $.tmpl(tmplContainerId, o).appendTo(self.element);
                        // Init *map*.
                        centerMarker = new window.google.maps.LatLng(o.lat, o.lng);
                        center = o.center ? new window.google.maps.LatLng(o.center.lat, o.center.lng) : centerMarker;
                        self.marker = new window.google.maps.Marker({
                            position: centerMarker
                        });
                        if (o.title) {
                            self.marker.setTitle(o.title);
                        }
                        self.map = new window.google.maps.Map(self.container[0], {
                            disableDefaultUI: !o.controls,
                            draggable: !o.fixedView,
                            center: center,
                            mapTypeId: o.mapType || 'roadmap',
                            zoom: o.zoom
                        });
                        self.marker.setMap(self.map);
                        if (typeof o.callback === 'function') {
                            o.callback(o.type, self.map, self.marker);
                        }
                    }
                });
            }
        },

        // Bing Map load and render API
        bing: {
            // Load API and return deferred object or false
            load: function () {
                var url = '//ecn.dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=6.2' + (ep.config.requestProtocolAndServer ? '&s=1' : '');

                // API already loaded
                if (window.VEMap) {
                    return;
                }

                // Request already started
                if (ajaxCache[url]) {
                    return ajaxCache[url];
                }

                // Request API
                return (ajaxCache[url] = ep.ajax({
                    url: url,
                    dataType: 'script',
                    cache: true
                }));
            },
            // Render map using loaded api
            render: function () {
                var self = this,
                    o = self.options,
                    uid = $.uid(),
                    map,
                    center,
                    interval,
                    initBingMap = function () {
                        map.LoadMap(center, o.zoom, o.mapType || window.VEMapStyle.Road, o.fixedView ? true : false, o.mode ? window.VEMapMode[o.mode] : window.VEMapMode.Mode2D);
                        if (!o.controls) {
                            map.HideDashboard();
                        }
                        self.marker = map.AddPushpin(center);
                        if (o.title) {
                            self.marker.SetDescription(o.title);
                        }
                        if (typeof o.callback === 'function') {
                            o.callback(o.type, self.map, self.marker);
                        }
                    };
                // Init *container*.
                self.container = $.tmpl(tmplContainerId, o).attr('id', uid).appendTo(self.element);
                // Init *map*.
                map = self.map = new window.VEMap(uid);
                center = new window.VELatLong(o.lat, o.lng);
                if ($.browser.mozilla) {
                    // If firefox is detected, use a setInterval to check that the window.VEMap exists and add an additional check to wait until attachEvent exists.
                    interval = setInterval(function () {
                        if (typeof window.VEMap !== "undefined" && document.getElementById(uid).attachEvent !== undefined) {
                            clearInterval(interval);
                            initBingMap();
                        }
                    }, 10);
                } else {
                    initBingMap();
                }
            },
            // Destroy rendered map
            destroy: function () {
                this.map.Dispose();
            }
        },

        // KlickTel Map load and render API
        klicktel: {
            // Load API and return deferred object or false
            load: function () {
                var klicktel = 'de_epages.externalcontent.ui.klicktel';

                if (!$.template[klicktel]) {
                    $.template(klicktel, '<iframe src="?' + 'ViewAction=ViewKlicktelMap&ChangeAction=CalculateKlicktelMap' + '&ObjectID=${epConfig.siteId}&Address=${address}&MapWidth={{if autoWidth}}auto{{else}}${width}{{/if}}&MapHeight=${height}' + '{{if contactPerson}}&ContactPerson=${contactPerson}{{/if}}{{if email}}&EMail=${email}{{/if}}&UseRoute={{if useRoute}}1{{else}}0{{/if}}' + '&LargeNavigation={{if largeNavigation}}1{{else}}0{{/if}}' + '&ShowAddress={{if showAddress}}1{{else}}0{{/if}}&LayoutLR={{if layoutLR}}1{{else}}0{{/if}}" ' + 'width=${width} height=${height} frameborder=0></iframe>');
                }

                return;
            },
            // Render map using loaded api
            render: function () {
                var self = this;

                self.container = $.tmpl('de_epages.externalcontent.ui.klicktel', self.options).appendTo(self.element);
            }
        },

        destroy: function () {
            // Remove all traces of the widget.
            var self = this,
                o = self.options;

            // Remove gadget class name
            self.element.removeClass(gadgetClassName);

            // Call destroy of map type
            if (self.api.destroy) {
                self.api.destroy.call(self);
            }

            // Remove container
            if (self.container) {
                self.container.remove();
            }

            self._superApply(arguments);
        }

    });

	return de_epages;

});