/*jslint nomen: true, white: true*/
/*global define*/

define("de_epages/mediagallery/filemanager/folder-view", [
    "jquery",
    "backbone",
    "$dict!../dictionary",
    "$tmpl!./folder-view",

    "ep/ui/tooltip"
], function ($, Backbone, mediagalleryDict, template) {
    "use strict";

    var tOpen                   = mediagalleryDict.translate("Open"),
        tDirectoryAlreadyExists = mediagalleryDict.translate("DirectoryAlreadyExists"),

        view = Backbone.View.extend({
            tagName:   "li",
            className: "ep-mediagallery-folderContainer",
            template:  template,

            events: {
                "dblclick" : "openFolder",
                "click .ep-mediagallery-itemName" : "edit",
                "keyup input" : "onKeyup",
                "blur input" : "save"
            },

            initialize: function(){
                this.model
                    // render view when an attribute changes
                    .on("change", this.render, this)
                    // display a tooltip when a folder with this name already exists
                    .on("directoryAlreadyExists", this.directoryAlreadyExists, this)
                    // show an input the edit the name of the folder
                    .on("editName", this.edit, this);
            },

            render: function(){
                var tmpl = this.template({
                            name: this.model.get("data"),
                            icon: this.model.get("icon"),
                            tOpen: tOpen
                        });

                $.extend(this, $.tmplItem(tmpl).elements);

                this.$el
                    .html(tmpl.children())
                    .data("itemID", this.model.get("id"));

                return this;
            },

            edit: function(){
                if(this.model.get("metadata").type === "MediaGallery"){
                    this.itemName.replaceWith('<input/>');
                    this.input = this.$('input');
                    this.input.val(this.model.get("data")).select();
                }
            },

            onKeyup: function(e){
                if(e.which === 13 || e.which === 27){
                    this.save(e);
                }
            },

            save: function(e){
                if(e.type === "keyup"){
                    this.keyup = true;
                }
                else if(e.type === "focusout" && this.keyup){
                    this.keyup = false;
                    return;
                }

                if((e.target.value === this.model.get("data")) || (e.target.value === "")){
                    if($(e.target).hasClass("DialogError")){
                        return;
                    }
                    this.render();
                }
                else{
                    this.model.save(
                        {data: e.target.value},
                        {
                            wait:   true,
                            method: "renameDirectory",
                            source: this.model.get("directory"),
                            view:   this
                        }
                    );
                }
            },

            directoryAlreadyExists: function(){
                this.itemName.trigger("click");
                this.input.addClass("DialogError");

                if(this.tooltip){
                    this.tooltip.uiTooltip('show');
                }
                else{
                    this.tooltip = $('<span class="Tooltip">'+ tDirectoryAlreadyExists +'</span>')
                        .uiTooltip({
                            context:     this.input,
                            orientation: "right"
                        });
                }
            },

            openFolder: function(event){
                if(event.target === this.$('input')[0]){
                    return;
                }
                this.trigger("openFolder", this.model.get("metadata"));
            }
        });

    return view;
});