/*global define*/

define("de_epages/mediagallery/filemanager/sync", ["ep"], function (ep) {
    "use strict";

    var sync = function (method, model, options) {

            var createDirectory = function () {
                    ep.ajax({
                        dataType: "json",
                        type: "POST",
                        data: {
                            ChangeAction: "JSONCreateDirectory",
                            DirectoryName: model.get("data"),
                            ObjectID: ep.config.siteId,
                            TargetDirectory: options.directory,
                            ViewAction: "JSONViewResponse"
                        }
                    }).done(function (jsonData, statusText, jqXHR) {
                        var data = jsonData[0].children[0];

                        data.id = data.metadata.type + data.metadata.path;

                        model.set(data).trigger("editName");

                        options.view.trigger("updateUsage", jsonData[0].currentGallerySize).trigger("addTreeNode", {
                            data: jsonData[0].children[0],
                            id: options.view.model.get("id"),
                            path: options.view.model.get("metadata").path
                        });
                    }).fail(function (jqXHR, statusText, errorThrown) {
                        if (errorThrown[0].Reason === "DirectoryAlreadyExists") {
                            model.trigger("directoryAlreadyExists");
                        }
                    });
                },

                renameDirectory = function () {
                    ep.ajax({
                        dataType: "json",
                        type: "POST",
                        data: {
                            ChangeAction: "JSONRenameDirectory",
                            SourceDirectory: model.get("metadata").path,
                            ObjectID: ep.config.siteId,
                            TargetDirectoryName: model.get("data"),
                            ViewAction: "JSONViewResponse"
                        }
                    }).done(function (jsonData, statusText, jqXHR) {
                        var prevID = model.get("id");

                        model.set({
                            data: jsonData.ResultingDirectory,
                            id: "MediaGallery" + jsonData.ResultingPath,
                            metadata: {
                                path: jsonData.ResultingPath,
                                type: "MediaGallery"
                            }
                        });
                        options.view.trigger("renameTreeNode", {
                            model: model,
                            prevID: prevID
                        });
                    }).fail(function (jqXHR, statusText, errorThrown) {
                        if (errorThrown[0].Reason === "DirectoryAlreadyExists") {
                            model.trigger("directoryAlreadyExists");
                        }
                    });
                },

                renameFile = function () {
                    ep.ajax({
                        dataType: "json",
                        type: "POST",
                        data: {
                            ChangeAction: "JSONRenameFile",
                            SourceFile: options.prevName,
                            ObjectID: ep.config.siteId,
                            TargetFileName: model.get("name"),
                            ViewAction: "JSONViewResponse"
                        }
                    }).done(function (jsonData, statusText, jqXHR) {
                        model.set({
                            name: jsonData.ResultingFilename,
                            placeholder: jsonData.ResultingFullPath,
                            path: jsonData.ResultingPath,
                            fullpath: jsonData.ResultingFullPath
                        });
                    }).fail(function (jqXHR, statusText, errorThrown) {
                        if (errorThrown[0].Reason === "FileAlreadyExists") {
                            model.trigger("fileAlreadyExists");
                        }
                    });
                },

                deleteFile = function () {
                    ep.ajax({
                        dataType: "json",
                        type: "POST",
                        data: {
                            ChangeAction: "JSONDeleteFile",
                            File: model.get("path"),
                            ObjectID: ep.config.siteId,
                            ViewAction: "JSONViewResponse"
                        }
                    }).done(function (jsonData, statusText, jqXHR) {
                        options.view.trigger("updateUsage", jsonData.currentGallerySize).trigger("removeExistingFile", model.get("name"));
                    });
                },

                deleteDirectory = function () {
                    ep.ajax({
                        dataType: "json",
                        type: "POST",
                        data: {
                            ChangeAction: "JSONDeleteDirectory",
                            Directory: model.get("metadata").path,
                            ObjectID: ep.config.siteId,
                            ViewAction: "JSONViewResponse"
                        }
                    }).done(function (jsonData, statusText, jqXHR) {
                        options.view.trigger("updateUsage", jsonData.currentGallerySize).trigger("removeTreeNode", {
                            id: model.get("id")
                        });
                    });
                };

            switch (options.method) {
            case "renameFile":
                renameFile();
                break;
            case "renameDirectory":
                renameDirectory();
                break;
            case "createDirectory":
                createDirectory();
                break;
            case "deleteFile":
                deleteFile();
                break;
            case "deleteDirectory":
                deleteDirectory();
                break;
            }
        };

    return sync;
});