/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.mediagallery.ui.filemanagerdialog
 *
 * @revision		$Revision: 1.6 $
 */

/*jslint nomen:true*/
/*global define*/

define("de_epages/mediagallery/ui/filemanagerdialog", [
	"jquery/ui/widget",
	"de_epages",
	"$dict!de_epages/mediagallery/dictionary",
	"ep/ui/dialog",
	"de_epages/mediagallery/ui/filemanager"
], function ($, de_epages, mediagalleryDict) {
	"use strict";
    /*
     *
     * @dictionary		de_epages.mediagallery.dictionary
     *
     * @translation		{FileRepository}
     *
     */

    var tFileRepository = mediagalleryDict.translate("FileRepository");

    $.widget("ui.mediagalleryUiFilemanagerdialog", {

        options: {
            dialog: {
                title: tFileRepository,
                width: 920,
                height: 563,
                draggable: false,
                modal: true,
                autoOpen: false
            },
            filemanager: {
                accept: "*/*",
                selectable: "",
                currentFolder: "MediaGallery"
            }
        },

        _create: function () {
            var self = this,
                o = self.options;

            self.dialog = $('<div class="fileManagerDialog"></div>').appendTo("body").uiDialog(o.dialog);

            self.element.on("click.mediagalleryUiFilemanagerdialog", function () {
                self.open();
            });

        },

        open: function () {
            if (!this.filemanager) {
                this._createFilemanager();
            }
            $(this.dialog).uiDialog("open");
        },

        close: function () {
            $(this.dialog).uiDialog("close");
        },

        destroy: function () {
            if (this.dialog) {
                this.dialog.remove();
                this.element.off("click.mediagalleryUiFilemanagerdialog");
            }

            this._superApply(arguments);
        },

        _createFilemanager: function () {
            var self = this,
                o = this.options;

            self.filemanagerContainer = $('<div class="fileManager"></div>').appendTo(self.dialog);
            self.filemanager = self.filemanagerContainer.mediagalleryUiFilemanager(o.filemanager);

            // trigger event if filemanager is ready
            self.filemanager.on("filemanagerReady", function () {
                self.element.trigger("filemanagerReady");
            });
        },

        resetFilemanager: function () {
            this.filemanager.mediagalleryUiFilemanager("destroy");
            this.filemanager = undefined;
        },

        getCurrentUsage: function () {
            return this.filemanager.mediagalleryUiFilemanager("getCurrentUsage");
        },

        getSelectedElements: function () {
            return this.filemanager.mediagalleryUiFilemanager("getSelectedElements");
        }
    });

    return de_epages;
});