use Test::More;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File  qw( GetSubDirectories ExistsDirectory GetInfoDirectory fsplit );

local $| = 1;

# test all subdirectories of $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/<CartridgeName>/Data/Public/
# for a directory named GUI wich does not exists anymore

# test all subdirectories of $ENV{EPAGES_CARTRIDGES}/DE_EPAGES/<CartridgeName>/Data/Public/lib
# for lowercase directory and filenames
# Exclusion lists are: $aDirectoryIgnoreList and $aFileExtensionIgnoreList
# you can use regexp statements in the $aDirectoryIgnoreList

sub TestCartridgeDirectories {

    #FileExtensionIgnoreList and aDirectoryIgnoreList starts after "$CartridgeDirectory/Data/Public/lib"

    my $aDirectoryIgnoreList = ['CVS','ckeditor','i18n'];
    my $aFileExtensionIgnoreList = ['.xml'];

    my $aCartridgeDirs = GetSubDirectories("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES", {'recursive' => 0});
    # test if a directory "GUI" exists  --> error
    foreach my $CartridgeDirectory (@$aCartridgeDirs) {
        ok( !ExistsDirectory("$CartridgeDirectory/Data/Public/GUI"),"$CartridgeDirectory/Data/Public/GUI does not exists");
    }

    # test if we have only lowercase filenames and directory names in subdirectory "/Data/Public/lib"
    my $TestCount = scalar @$aCartridgeDirs;
    foreach my $CartridgeDirectory (@$aCartridgeDirs) {

        my $LibDir = "$CartridgeDirectory/Data/Public/lib";

        if (ExistsDirectory($LibDir)) {

            my $hInfoDir = GetInfoDirectory( $LibDir , { relative => 1, recursive => 1 } );
            # test if we have only lowercase directory names and files
            foreach my $Name (@{$hInfoDir->{'Files'}}, @{$hInfoDir->{'Directories'}}) {
                #test for ignore directories
                my $ig = 0;
                foreach my $DirIgnore (@$aDirectoryIgnoreList){
                    $ig = 1 if $Name =~ /$DirIgnore/;
                }
                next if $ig;
                next unless $Name;

                # ignore files from ignorelist
                my $Extension = (fsplit($Name))[2];
                next if $Extension ~~ $aFileExtensionIgnoreList;
                #print "$Path, $FileName, $Extension\n";

                $TestCount ++;
                my $LCName = lc($Name);
                ok( $Name eq $LCName,"lowercase file and directory names: $Name in path: $LibDir");

            }
        }
    }

    return $TestCount ;
}

RunScript(Debug=>1, Sub=> sub {
    my $TestCount = TestCartridgeDirectories();
    # set the counter here because we don't know how much directories and files we have
    plan tests => $TestCount;
});

